-- coding: utf-8
local WoWMapPOI=_G.WoWMapPOI

function WoWMapPOI.Frame_OnShow(this)

    this.data = this.data or {}
    local data = this.data

    local zname = WoWMap.GetZoneNameLocal(data.map or 1) or ""
    local text = string.format("%.1f / %.1f - %s", (data.x or 0.5)*100, (data.y or 0.5)*100,zname)
    WoWMapPOIFramePosition:SetText(text)
    WoWMapPOIFrameDescTitle:SetText(data.desc or "")
    WoWMapPOIFrameEditText:SetText(data.text or "")
    WoWMapPOIFrameNpcID:SetText(data.npc and data.npc.id or "")
    WoWMapPOI.ShowNPCInfoByID(data.npc and data.npc.id or 0)

    if data.npc.isunique then
        WoWMapPOIFrameNpcName:SetText(data.npc.cur_name)
    end

    -- setup icons
    local icondrop = _G[this:GetName().."IconDropDown"]
    icondrop.icons = {}
    for i, texdata in ipairs(WoWMapPOI.CustomNodes.textures) do
        table.insert(icondrop.icons, texdata)
    end
    WoWMapPOI.IconChooseSelectIcon(icondrop, data.icon or 1)

    WoWMapPOIFrameDescTitle:SetFocus()
    WoWMapPOIFrameDescTitle:SetPlaceCaret(-1)

    if data.edit_mode then
        WoWMapPOIFrameShare:Hide()
        WoWMapPOIFrameDelete:Show()
        WoWMapPOIFrameAdd:Show()
        WoWMapPOIFrameAdd:SetText(TEXT("MACRO_MODIFY"))
    else
        WoWMapPOIFrameShare:Hide()
        WoWMapPOIFrameDelete:Hide()
        WoWMapPOIFrameAdd:Show()
        WoWMapPOIFrameAdd:SetText(TEXT("MACRO_CREATE"))
    end

    --@non-debug@
        WoWMapPOIFrameNpcID:Hide()
        WoWMapPOIFrameNpcName:Hide()
        WoWMapPOIFrameNpcSelect:Hide()
        WoWMapPOIFrameNpcPos:Hide()

        WoWMapPOIFramePosition:ClearAllAnchors()
        WoWMapPOIFramePosition:SetAnchor("TOP", "TOP", "WoWMapPOIFrame", 0, 87)
    --@end-non-debug@
end

function WoWMapPOI.IconChooseSelectIcon(this,id)
    local texture = _G[this:GetName().."Icon"]
    local texdata = this.icons[ id or 1]
    texture:SetTexture(texdata[1])
    texture:SetTexCoord(texdata[2],texdata[4],texdata[3],texdata[5])
end

function WoWMapPOI.LayoutIcons(this)
    local icons = this:GetParent().icons
    local count=#icons
    local sw=math.floor(math.sqrt(count))
    local sh = sw
    if sw*sh<count then sh =sh+1 end

    this:SetSize(10+sw*30,10+sh*30)

    for i=1,29 do
        local name=this:GetName().."But"..i
        _G[name]:Hide()
    end

    for i=1,count  do
        local x= (i-1)%sw
        local y= math.floor((i-1)/sw)
        local name=this:GetName().."But"..i

        local texture = _G[name.."Icon"]
        local texdata = icons[i]
        texture:SetTexture(texdata[1])
        texture:SetTexCoord(texdata[2],texdata[4],texdata[3],texdata[5])

        local but = _G[name]
        but:ClearAllAnchors()
        but:SetAnchor("TOPLEFT", "TOPLEFT", this, 5+x*30, 5+y*30)
        but:Show()
    end
end

function WoWMapPOI.IconPicked(this, key)
    local chooser = this:GetParent()
    local ctrl = chooser:GetParent()
    chooser:Hide()

    local id = this:GetID()
    WoWMapPOIFrame.data.icon = id
    WoWMapPOI.IconChooseSelectIcon(ctrl,id)
end

function WoWMapPOI.Frame_OnHide(this)
    _G[this:GetName().."IconDropDownChooser"]:Hide()
end

function WoWMapPOI.UpdateNPCInfo(new_pos)

    local npcinfo = WoWMapPOIFrame.data.npc

    local count = 0
    if npcinfo.byID then
        count = NpcTrack_SearchNpcByDBID( npcinfo.id )
    else
        count = NpcTrack_SearchNpc( npcinfo.name )
    end

    npcinfo.nr = new_pos or npcinfo.nr
    if not npcinfo.nr then
        npcinfo.nr = WoWMapPOI.FindBestNPCNr(count)
    elseif npcinfo.nr < 1 then
        npcinfo.nr = 1
    elseif npcinfo.nr > count then
        npcinfo.nr = count
    end

    if count and count >0 then
        local npcID, n_name, mapID, x, y = NpcTrack_GetNpc(npcinfo.nr)
        local zname = WoWMap.GetZoneNameLocal(mapID) or ""
        if mapID ~= WoWMapPOIFrame.data.map then
            zname = "|cffff4040"..zname.."|r"
        end
        WoWMapPOIFrameNpcPos:SetText(string.format("%s - %.2f / %.2f - %s",n_name, x*100,y*100,zname))
        npcinfo.cur_id = npcID
        npcinfo.cur_name = n_name
        npcinfo.isunique = true

        WoWMapPOIFrameNpcSelect:Hide()
        if count>1 then
            npcinfo.isunique = false
            WoWMapPOIFrameNpcSelect:Show()
            WoWMapPOIFrameNpcSelectVal:SetText(string.format("%i / %i",npcinfo.nr,count))
        end

        if not npcinfo.byID then
            WoWMapPOIFrameNpcID:SetText(npcID)
        end

    else
        WoWMapPOIFrameNpcSelect:Hide()
        WoWMapPOIFrameNpcPos:SetText("")
        npcinfo.cur_id=nil
        npcinfo.cur_name=nil
    end
end

function WoWMapPOI.FindBestNPCNr(count)
    for i=1,count do
        local npcID, n_name, mapID, x, y = NpcTrack_GetNpc(i)
        if mapID == WoWMapPOIFrame.data.map then return i end
    end

    return 1
end


function WoWMapPOI.NpcNameChanged(text)
    local npcinfo = WoWMapPOIFrame.data.npc
    if npcinfo and npcinfo.byID==false and npcinfo.name == text then
        return
    end

    WoWMapPOIFrame.data.npc = {
        byID=false,
        name=text
        }

    WoWMapPOI.UpdateNPCInfo()

    if WoWMapPOIFrame.data.npc.cur_id then
        WoWMapPOIFrameNpcID:SetText(WoWMapPOIFrame.data.npc.cur_id)
    else
        WoWMapPOIFrameNpcID:SetText("?")
        WoWMapPOIFrame.data.npc = nil
    end
end

function WoWMapPOI.NpcIDChanged(text)
    local num = tonumber(text)
    if num then
        WoWMapPOI.ShowNPCInfoByID(num)
    else
        WoWMapPOIFrame.data.npc.cur_id = nil
    end
end

function WoWMapPOI.ShowNPCInfoByID(npc_id)
    WoWMapPOIFrame.data.npc = {
        byID=true,
        id=npc_id
        }

    WoWMapPOI.UpdateNPCInfo()

    if WoWMapPOIFrame.data.npc.cur_name then
        WoWMapPOIFrameNpcName:SetText(WoWMapPOIFrame.data.npc.cur_name)
    else
        WoWMapPOIFrameNpcName:SetText("?")
    end
end

function WoWMapPOI.Frame_SavePOI()
    if WoWMapPOIFrame.data.edit_mode then
        WoWMapPOI.Frame_DeletePOI()
    end

    if WoWMapPOIFrame.data then
        local text = WoWMapPOIFrameDescTitle:GetText().."\n"..WoWMapPOIFrameEditText:GetText()
        local data = WoWMapPOIFrame.data
        WoWMapPOI.CustomNodes.maps[data.map] = WoWMapPOI.CustomNodes.maps[data.map] or {}
        table.insert(WoWMapPOI.CustomNodes.maps[data.map],{x=data.x,y=data.y,npc=data.npc.cur_id,tip=text,icon=data.icon,npc_is_unique=data.npc.isunique})
        WoWMap.ShowPOIs()
    else
        DEFAULT_CHAT_FRAME:AddMessage("Error creating POI",1,0,0)
    end
    WoWMapPOIFrame:Hide()
end

function WoWMapPOI.Frame_DeletePOI()
    if WoWMapPOIFrame.data then
        local data = WoWMapPOIFrame.data
        local mdata = WoWMapPOI.CustomNodes.maps[data.map]
        if mdata then
            for i,d in ipairs(mdata) do
                if d.x==data.x and d.y==data.y then
                    table.remove(mdata,i)
                end
            end

            if #mdata==0 then
                WoWMapPOI.CustomNodes.maps[data.map]=nil
            end
        end
        WoWMap.ShowPOIs()
    else
        DEFAULT_CHAT_FRAME:AddMessage("Error creating POI",1,0,0)
    end
    WoWMapPOIFrame:Hide()
end

function WoWMapPOI.ModifyContextMenu( this )
    local info = {}
    info.text 	= "----------------"
    info.value	= 0
    info.owner	= DropDownframe
    UIDropDownMenu_AddButton( info, 1 )

    info = {}
    info.text 	= WoWMapPOI.Text.CreatePOI
    info.value	= 0
    info.func 	=
        function()
            WoWMapPOI.CreateNewPOI( this.mapID, this.mapX, this.mapY )
        end
    info.owner	= DropDownframe
    UIDropDownMenu_AddButton( info, 1 )

    if WoWMapPOI.ClickedOnCustomIcon() then
        info = {}
        info.text 	= WoWMapPOI.Text.EditPOI
        info.value	= 0
        info.func 	=
            function()
                WoWMapPOI.EditPOI( WoWMap.Data.CurrentIcon )
            end
        info.owner	= DropDownframe
        UIDropDownMenu_AddButton( info, 1 )
    end
end

function WoWMapPOI.ClickedOnCustomIcon()
    return WoWMap.Data.CurrentIcon and WoWMap.Data.CurrentIcon.POI.name==WoWMapPOI.Text.MyPOIs
end

function WoWMapPOI.CreateNewPOI(m,px,py)
    px = math.floor((px or 0.5)*1000)/1000
    py = math.floor((py or 0.5)*1000)/1000
    WoWMapPOIFrame.data={map=m,x=px or 0.5,y=py or 0.5}
    WoWMapPOIFrame:Show()
end

function WoWMapPOI.EditPOI( icon )
    assert(icon.data)

    local lines=WoWMap.Split(icon.data.tip or {}, "\n")

    WoWMapPOIFrame.data= {
        map = icon.map,
        x   = icon.data.x,
        y   = icon.data.y,
        icon= icon.data.icon,
        npc = {id=icon.data.npc},
        desc= lines[1],
        text= table.concat(lines, "\n", 2),
        edit_mode=true,
    }
    WoWMapPOIFrame:Show()
end

SLASH_WOWMapPoi1 = "/poi";
SlashCmdList["WOWMapPoi"] = function (editBox, msg)
    local pmap = WoWMap.GetCurrentWorldMapID()
    local px,py = WoWMap.GetPlayerPos(pmap)

    if pmap then

        -- test last NPC
        local npc_id = nil
        if WoWMap.last_npc then
            local count = NpcTrack_SearchNpcByDBID( WoWMap.last_npc )
            for i=1, count do
        		local npcID, name, mapID, x, y = NpcTrack_GetNpc(i)
                local dist = (px-x)*(px-x)+(py-y)*(py-y)
                if mapID == pmap and dist<(0.005*0.005*2) then
                    npc_id = npcID
                end
	        end
        end



        WoWMapPOIFrame.data={map=pmap,x=px or 0.5,y=py or 0.5,npc={id=npc_id},desc=msg}

        ShowUIPanel( WorldMapFrame )
        WorldMapFrame_SetWorldMapID(pmap)
        WoWMapPOIFrame:Show()

        if msg and msg~="" then
            WoWMapPOI.Frame_SavePOI()
            HideUIPanel( WorldMapFrame )
        end

    end
end




-----------------------------
WoWMapPOI_Nodes = {}

WoWMapPOI.CustomNodes = {
    name=WoWMapPOI.Text.MyPOIs,
    textures={
        {"interface/addons/wowmap-poi/images/poi_customs.dds", 0/8, 0/4,1/8,1/4},
        {"interface/addons/wowmap-poi/images/poi_customs.dds", 1/8, 0/4,2/8,1/4},
        {"interface/addons/wowmap-poi/images/poi_customs.dds", 2/8, 0/4,3/8,1/4},
        {"interface/addons/wowmap-poi/images/poi_customs.dds", 3/8, 0/4,4/8,1/4},
        {"interface/addons/wowmap-poi/images/poi_customs.dds", 4/8, 0/4,5/8,1/4},
        {"interface/addons/wowmap-poi/images/poi_customs.dds", 0/8, 1/4,1/8,2/4},
        {"interface/addons/wowmap-poi/images/poi_customs.dds", 1/8, 1/4,2/8,2/4},
        {"interface/addons/wowmap-poi/images/poi_customs.dds", 2/8, 1/4,3/8,2/4},
        {"interface/addons/wowmap-poi/images/poi_customs.dds", 3/8, 1/4,4/8,2/4},
        {"interface/addons/wowmap-poi/images/poi_customs.dds", 4/8, 1/4,5/8,2/4},
        {"interface/addons/wowmap-poi/images/poi_customs.dds", 5/8, 1/4,6/8,2/4},
        {"interface/addons/wowmap-poi/images/poi_customs.dds", 6/8, 1/4,7/8,2/4},

        },
    size={16,16},
    priority=100,
    maps={},
}

function WoWMapPOI.UpdateCustomNodes()
    WoWMap.ShowPOIs()
end

function WoWMapPOI.CleanUpCustomNodes()
    for id,data in pairs(WoWMapPOI.CustomNodes.maps) do
        if not next(data) then
            WoWMapPOI.CustomNodes.maps[id]=nil
        end
    end
end
